function Plant = fun_plant_setup
% FUN_PLANT_SETUP   Sample plant setup file for initialization of Plant structure.
%   Plant = fun_plant_setup
% where
% Plant is a structre where fhe following fields are set:
%   funf    name of the DSS (discrete-time state space) system equation m-file
%   funh    name of the measurement equation m-file
%   Ts      system sampling time
%   Xref    state discretization (reference points)
%   Uref    control discretization (action space)
%   Yref    output discretization (measurement reference points)
%   Wref    set point discretization

% See FUNF, FUNH

% DSS system info
% ---------------
funf_str = 'funf'  % name of m-file containing system equations x(k+1) = f(x(k),u(k))
funh_str = 'funh'   % name of m-file containing measurement equations y(k) = h(x(k))
Ts = eval(funf_str);          % read Ts from FUNF
Plant.funf = funf_str;
Plant.funh = funh_str;
Plant.Ts = Ts;

% Discretize variables:
% ---------------------
% read info from FUNF and FUNH
[Ts,xn,un,xmin,xmax,umin,umax,xname,uname] = eval(funf_str);  % read info from FUNF
[yn,ymin,ymax,yname] = eval(funh_str);  % read info from FUNH

% Write variable names to Plant-structure
Plant.Xname = xname;
Plant.Uname = uname;
Plant.Yname = yname;
Plant.Wname = yname;

% read sizes of variables
n_x = size(xn,1);
n_u = size(un,1);
n_y = size(yn);
n_w = n_y;

% Discretize spaces
if 0
    dx = 0.05;
    for i=1:n_x, x_part{i} = [-0.5:dx:2.5]; end
    for i=1:n_u, u_part{i} = [0:0.05:1]; end
    for i=1:n_y, y_part{i} = [-0.5:dx:2.5]; end
    for i=1:n_w, w_part{i} = [0:2*dx:2]; end    % set target discretization to measurement discretization
else
    % An example of grid discretization
    % where # discretized values is given for each dimension
    % and sets with equidistant elements are generated 
    % using min-max ranges for each variable / dimension
    
    % number of discretizations (user defined)
    Si = 100;     % for each dimension in x
    %Si = 300;     % for each dimension in x
    Su = 10;     % for each dimension in u
    Sy = 100;     % for each dimension in y
    Sw = 10;     % number of setpoints
    
    % generate discretization for each dimension
    for i=1:n_x, x_part{i} = eqspace(xmin(i),xmax(i),Si); end
    for i=1:n_u, u_part{i} = eqspace(umin(i),umax(i),Su); end
    for i=1:n_y, y_part{i} = eqspace(ymin(i),ymax(i),Sy); end
    for i=1:n_w, w_part{i} = eqspace(ymin(i),ymax(i),Sw); end
    %w_part = y_part;    % set target discretization to measurement discretization
end

% generate cmd-string for using 'gcm_inigrid', evaluate

% Define Xref
cmd = 'gcm_inigrid(';
for i=1:n_x, cmd = [cmd,sprintf('x_part{%d},',i)]; end; cmd = [cmd(1:length(cmd)-1),');'];
Xref = eval(cmd);

% Define Uref
cmd = 'gcm_inigrid(';
for i=1:n_u, cmd = [cmd,sprintf('u_part{%d},',i)]; end; cmd = [cmd(1:length(cmd)-1),');'];
Uref = eval(cmd);

% Define Yref
cmd = 'gcm_inigrid(';
for i=1:n_y, cmd = [cmd,sprintf('y_part{%d},',i)]; end; cmd = [cmd(1:length(cmd)-1),');'];
Yref = eval(cmd);

% Define Wref
cmd = 'gcm_inigrid(';
for i=1:n_w, cmd = [cmd,sprintf('w_part{%d},',i)]; end; cmd = [cmd(1:length(cmd)-1),');'];
Wref = eval(cmd);

% write to Plant
Plant.Xref = Xref;
Plant.Uref = Uref;
Plant.Yref = Yref;
Plant.Wref = Wref;

% === SUBFUN EQSPACE ======================
function x=eqspace(xmin,xmax,n)
if n>1
    x=[xmin:(xmax-xmin)/(n-1):xmax];
elseif n==1
    x=(xmin+xmax)/2;
else
    error('EQSPACE: Must have n>0')
end
