function [xk1,o2,o3,o4,o5,o6,o7,o8,o9] = funf(xk,uk,Ts)
% FUNF  Sample state equation m-file.
%
% Syntax for initial info (no input arguments):
%   [Ts,Xnom,Unom,Xmin,Xmax,Umin,Umax,Xnames,Unames] = funf; 
% where:
%   Ts                  sampling time [1,1]
%   Xnom, Unom          system nominal values [n_x,1], [n_u,1]
%   Xmin,Xmax,Umin,Umax system min-max range
%   Xname,Uname         string-names for elements in x and u
%                           Xname{1} = 'name of x_1', etc.
%
% Syntax for DSS state equation:
%   Xk1 = funf(Xk, Uk, Ts)  (simulate system equations one step ahead)
% where:
%   Xk,Xk1  system state x(k+1), x(k) [n_x,N]
%   Uk      system control u(k) [n_u,1]
%   Ts      sampling time [1,1]
%   N       batch size (compute evolution from N initial states x(k) under u(k))
%
% FUNF is a sample file describing a second order non-minimum phase SISO system
% n_x = 2, n_u = 1, n_y = 1. See also FUNH.

% Enso 6/2007

% Set system matrices A,B,C
% -------------------------
if nargin<3, Ts = 1; end % default Ts
    
if Ts==1 % default Ts
    itau = -5;
    switch(itau)
        % Linear SISO DSS model itau = -5
    case -5        
        A = [0 1;-0.54881163609403   1.51136807774859];
        B = [-0.33171166313774 -0.09473963885467]';
        C = [1 0];
        D = 0;
    case 0
        A = [0    1.0000;   -0.5488    1.5114];
        B = [0.0412    0.0959]';
        C = [1     0];
        D = 0;
    end
    
else % user given Ts
    % Create linear DSS:    
    % original cont-time system
    k=2    % gain
    itau = -5  % inverse time constant
    tau1 = 2, tau2 = 10    % time constants
    sys1 = tf(k*[itau 1],conv([tau1 1],[tau2 1])) % cont-time tf 
    
    % discretization
    Ts = 1;
    sys2 = c2d(sys1,Ts);
    
    % state-space form
    sys3 = ss(sys2);        % convert to state space
    sys4 = canon_ob(sys3);  % convert to observable form with measured states
    [A,B,C,D]=ssdata(ss(sys4));  % read system matrices

    % testplot:
    % step(sys1,sys2+0.1,sys3+0.2,sys4+0.3); shg
end

% Nominal values
% --------------
if nargin==0
    % output nominal values
    Ts = 1;
    un = 0.5;
    n = length(A); k = inv(eye(n)-A)*B;   % steady state
    xn = k*un;
    % min-max ranges
    umin = 0; umax = 1; % control at 0...1
    sys = ss(A,B,C,D,Ts);
    U = (umax-umin)*rand(1000,1)+umin; [y,t,x] = lsim(sys,U);    % simulate to find min & max
    xmin = floor(min(x)'); xmax = ceil(max(x)');
    % name-strings
    [n_x,tmp] = size(xn);
    for j=1:n_x
        Xnames{j} = sprintf('state-%d',j);
    end
    Unames = {'manipulated variable'};

    % set output args
    xk1 = Ts; o2 = xn; o3 = un;

    o4 = xmin; o5 = xmax;
    o6 = umin; o7 = umax;
    o8 = Xnames; o9 = Unames;
    return;
end

% Prepare for batch version
% -------------------------
[n_x,N] = size(xk);
if N>1
    % expand B from column vector to a matrix with N columns
    for i=1:n_x, B(i,1:N)=B(i,1); end
end

% Solve system equations
% ----------------------
xk1 = A*xk + B*uk;
% yk1 = C*xk1;      % write this to 'funh'-mfile

