function [BofA, P_rr_inf, BofA_proj,In] = gcm_bofa_size(P,EA,P_t2r,Xref_proj)
% GCM_BOFA_SIZE  Estimate the size of basin-of-attraction. 
%
% Estimate the size of b-of-a:
%   [BofA_size,P_rr_inf] = gcm_bofa_size(P,EA,P_t2r)
% BofA_size Estimated size of basin-of-attraction for each recurrent state [1,lipr]
% P_rr_inf  Mapping to a stationary distribution
% EA        Structure from gcm_eiganalysis, field 'ipr' is used (indicates recurrent states)
% P_t2r     Probability of transition from transient to recurrent cell
%
% Projected size of b-of-a:
%   [BofA_size,P_rr_inf,BofA_proj,In] = gcm_bofa_size(P,EA,P_t2r,Xref_proj)   *
% will return the estimate of basin-of-attraction as projected to 
% the mapping in Xref_proj [1,S]
%
% Shorter commands:
%   BofA_size = gcm_bofa_size(P)
%   [BofA_size,P_rr_inf,BofA_proj] = gcm_bofa_size(P,Xref_proj)    *
% will evoke GCM_EIGANALYSIS to compute all that's necessary.
% *If no output arguments are given, a bar plot is produced.
%
% See GCM_EIGANALYSIS.

% If eigenanalysis info is not given, compute it: EA, P_t2r
% -----------------------------------------------

if nargin<3
    fprintf('GCM_BOFA: ')
    if nargin==2, Xref_proj=EA; end
    [EA,P_t2r] = gcm_eiganalysis(P);
end

% Compute mapping to stationary distribution for recurrent cells: P_rr_inf
% ---------------------------------------------------------------
% Proper computation / no missing values (may not work for large P)
% - steady-state map from elements of P
if nargin<3
    P_inf = gcm_pinf(P);
else
    % Create a dummy structure tmp with P as its field's field
    % (Needed in order to avoid recomputation of eigenvalues in gcm_pinf)  
    tmp.f(1).P = P;
    P_inf = gcm_pinf(tmp,1,EA,P_t2r);
end

%'average size for basin of attraction': BofA
% -------------------------------------------
ipr = EA.ipr;
tmp = sum(P_inf(ipr,:)');
% sum probabilities of entering to a particular cell (nonzero only for recurrent cells)
% However, there ipr obtained from eigenanalysis seems to differ some times from the above 'experimental' 
%  => We use the ipr obtained from eigenanalysis, and therefore have to normalize probabilities
S1 = full(sum(tmp));
S = size(P,1); tmp = S*tmp/sum(tmp);    % normalize
%tmp = sum(P_inf'); ipr2 = find(tmp>1);
if abs(S1-S)/S>0.01,    % warn about errors larger than 1%
    disp(sprintf('   GCM_BOFA_SIZE: Suspicious inconsistency in determination of recurrent cells (%d vs %1.1f.)',S,S1)); 
end
BofA = full(tmp);

% Extract stationary recurrent transition probabilities: P_rr_inf
% -------------------------------------------------------
P_rr_inf = P_inf(ipr,ipr);


% Compute projected sizes of basin-of-attraction, and plot them: BofA_proj
% ------------------------------------------------------------------------
if (nargout>1 | nargout==0) & (nargin==2 | nargin==3 | nargin==4)
    % cells to project to: 
    In = gcm_get_partitions(Xref_proj); % In: Xref_proj ordered, duplicates removed
    lIn = length(In);
    BofA_proj = zeros(1,lIn);
    % go through all different cells o in the list...
    for o=1:lIn
        % ... and check each recurrent cell r...
        for r=1:length(ipr)
            % ... for a match (i.e. look for all occurances of value In(o) in the list Xref_proj)
            if Xref_proj(ipr(r))==In(o)
                % Take a cumulative sum of the associated sizes of basin-of-attraction for these cells,
                BofA_proj(o) = BofA_proj(o) + BofA(r);
            end
        end
    end
    % plot
    if nargout==0
        bar(In,BofA_proj);
    end
end