function [CClass2,BofA_cells,Et_cells] = gcm_comm(i1,i2)
% GCM_COMM  Communicating classes and the associated basins of attraction.
%
%   CClass = gcm_comm(P)
%   CClass = gcm_comm(Plant,a)
%   CClass = gcm_comm(Plant_cl,q)
%
% P         probability transition matrix
% Plant     Plant-structure (or closed-loop structure Plant_cl)
% a         Plant input (or closed-loop set point q)
% CClass    An array of C vectors, where C is the number of communicating classes.
%           Each vector contains a list of recurrent cells that communicate with each others.
%
% The basin of attraction is obtained by
%   [CClass,BofA_cells,Et_cells] = gcm_comm(P,plim)
% where 
%   BofA_cells{c}   is the vector of indexes of cells that map to 
%                   the c'th class with probability > plim (default plim = 1e-3)
%   Et_cells{c}     is a vector of expected times of entering the c'th class 
%                   (weighted with probability of transition, P_t2r)
% Within each class, the cells are ordered from fastest to slowest.

% Example:
%   Plant_cl = gcmclosed(Plant,Cost);       % given 'Plant' and 'Cost', get closed-loop GCM
%   q = 3;                                  % consider 3rd set point (if so many exist..)
%   P = Plant_cl.f(q).P;                    % read the associated pr.trans. map
%   [CClass,BofA_cells,Et_cells] = gcm_comm(P);      % find comm. classes & b-of-a
%   c = 7;                                  % consider 7'th class (if so many exist..)
%   s0 = BofA_cells{c}(4)                   % take 4'th state in that class' b-of-a (if so many exist..)
%   figure(1); gcm_free(Plant,Cost,s0,q); 
%   figure(2); gcm_freek(Plant,Cost,s0,q);
%   sf = CClass{c};
%   Plant.Xref(:,s0), Plant.Xref(:,sf)
%   Et_cells{c}(4).*Plant.Ts                          
% The plots show that stating from state Plant.Xref(:,s0), the state
% trajectory converges to states Plant.Xref(:,sf) (with probability > plim)
% Expected time of entering that class is shown last.


% Enso 11/2006

% read input arguments
plim = 1e-3;
if nargin>1
    if isstruct(i1)
        P = i1.f(i2).P;
    else
        P = i1;
        plim = i2;
    end
else
    P = i1;
end

if nargout==2
    %[EA,P_t2r] = gcm_eiganalysis(P);
     [EA,P_t2r] = gcm_eiganalysis(P,[],50,1);
elseif nargout==3
    %[EA,P_t2r,Et_t2r] = gcm_eiganalysis(P);
     [EA,P_t2r,Et_t2r] = gcm_eiganalysis(P,[],50,1);
else
    %EA = gcm_eiganalysis(P);
    EA = gcm_eiganalysis(P,[],50,1);
end
ipa = EA.ipa;
ipr = EA.ipr;
ipt = EA.ipt;

P_rr = P(ipr,ipr);   % consider only recurrent states
S = length(P_rr);
s = 1; cl = 1; done=[];
Z = zeros(1,S);
while 1
    % -look for cells from where there is a nonzero 
    %  probability of entering the recurrent cell s
    nz = P_rr(s,:)>0; nz(s)=1; nz = logical(nz);
    lnz = length(nz);
    % find all nozero passages
    while 1
        % look for the cells that have a nonzero probability of entering these cells
        if sum(nz)==1
            nz = P_rr(nz,:)>0;
        else
            nz = logical(max(P_rr(nz,:)>0));
        end
        lnz_old = lnz;
        lnz = sum(nz);
        % if the number of cells remains the same, this is a closed set (communicating class)
        if lnz==lnz_old, break; end
    end
    % store
    CClass{cl} = ipr(find(nz));
    % mark as read
    done = [done, find(nz)];
    cl = cl+1;
    % next undread s
    while 1
        s = s+1;
        if ~any(find(s==done)), break; end
        if s>S, break; end
    end
    if s>S, break; end
end

% sort classes from largest to smallest
lCl = length(CClass);
for c=1:lCl, lc(c) = length(CClass{c}); end
[lc,idx] = sort(lc);
lc=lc(lCl:-1:1);
idx=idx(lCl:-1:1);
for c=1:lCl
    CClass2{c} = CClass{idx(c)};
end

lipa = length(ipa);
fprintf('GCM_COMM: Among %d recurrent states:\n   %d absorbing states \n   %d other communicating classes ',length(ipr),lipa,lCl-lipa);
if lCl
    fprintf('(max size %d)\n',length(CClass2{1}));
end

if 0
    if isstruct(Plant)
        % divide into absorbing classes, and classes larger than 1
        Cla = [];
        disp('Communicating classes:')
        disp('----------------------')
        for c=1:lCl
            if length(CClass2{c})>1
                gcm_dispref(CClass2{c},Plant.Xref);
            else
                Cla = [Cla;CClass2{c}];
            end
        end
        disp('');
        disp('Absorbing states:')
        disp('-----------------')
        gcm_dispref(Cla,Plant.Xref);
    end
end

if nargout>1

    % elements in the basin of attraction for each class
    % expected time of transition to each class
    
    % go through each class
    for c=1:length(CClass2)
        % get indexes to recurrent cells in each class, indexed among recurrent cells
        ir = i2i(1,2,CClass2{c},EA.ipr);
        ir = gcm_get_partitions(ir);    % remove duplicates
        
        % get list of transient cells that point (>plim) to recurrent cells in class c
        it = [];
        for j=ir
            % P_t2r is the static map!
            itx = find(P_t2r(j,:)>plim);     % list of transient cells that have nonzero p to enter class c
            it = [it, itx];
        end
        if length(it), it = gcm_get_partitions(it); end   % remove duplicates
        
        % list of all cells that are in the basin of attration of the c'th class
        islist = [CClass2{c}' EA.ipt(it)'];
        BofA_cells{c} = islist;
        
        if nargout>2
            % Expected time of transition from these cells to the class
            % Sum of Et_t2 among each class members, weighted by probability of transition
            % From recurrent cell the exp. time is zero
            lr = length(CClass2{c});
            idx_r = [1:lr];
            idx_t = [lr+1:length(islist)];
            Et_cells{c}(idx_r) = zeros(1,lr);
            if length(it), it = i2i(1,2,EA.ipt(it),EA.ipt); end
            if length(ir)>1
                Et_cells{c}(idx_t) = full(sum(P_t2r(ir,it).*Et_t2r(ir,it))./sum(P_t2r(ir,it)));
            else
                Et_cells{c}(idx_t) = full((P_t2r(ir,it).*Et_t2r(ir,it))./(P_t2r(ir,it)));
            end
            
            % sort lists according to speed of convergence
            [Et_cells{c},idx] = sort(Et_cells{c});
            BofA_cells{c} = BofA_cells{c}(idx);
        end
        
    end
end


% ----------------------------------------------------------
function j = i2i(fromset,toset,i,iplist)
% 1 - set of all cells: 1,2,...,S
% 2 - subset of cells: iplist
x = 10*fromset+toset;
switch(x)
case {11,22}
    j = i;
case 21
    % from  recurrent to all
    j = iplist(i);
case 12
    % from all to recurrent
    for k=1:length(i)
        j(k) = find(i(k)==iplist);
    end
end
    