function [EA,P_t2r,Et_t2r,Et_stat] = gcm_eiganalysis(P,a,K,PRECISION_FLAG)
%GCM_EIGENANALYSIS  Eigenvalue (and other) analysis of the state transition matrix.
%
%      EA = gcm_eiganalysis(P)
%      [EA,P_t2r,Et_t2r,Et_stat] = gcm_eiganalysis(Plant,a,<K>,<PF>)
%
% where:
%
% P         a state transition matrix [S,S], describing the mapping: p_{k+1} = P*p_k
% a         in the case of two input arguments: P=Plant.f(a).P.
% K         # eigenvectors to find (passed to GCM_EIG) (default K=100)
% PF        PrecisionFlag. If PF=1, GCM_EIG is repeated for transient cells 
%           until no new recurrent cells appear (default PF=1).
%
% EA is a structure with the following fields:
%   ipa       indexes to absorbing states [lipa,1]. Cells are never left when entered, (P(i,i)=1.
%   ipr       indexes to recurrent states [lipr,1]. Cells are entered infinitely often (include abs. cells).
%   ipt       indexes to transient states [lipt,1]. Cells are entered finitely often
%
% P_t2r     probability map for entering an recurrent cell, from a transient cell [lipr, lipt]
% Et_t2r    expected time of transition to an recurrent cell, from a transient cell [lipr, lipt]
% Et_stat is a structure with the following fields:
%   meanEt    average expected time of transition to an absorbing cell [lipr,1]
%   maxEt     largest expected time of transition to an absorbing cell [lipr,1]

% Enso 11/2006, 9/2007

% input argument P:
if nargin>1 & length(a)
    P = P.f(a).P;
else
    ; % enable use of input argument K for matrix inputs P
end
% input argument K:
mx = 100; if nargin>2, mx = K; end
% input argument PRECISION_FLAG:
if nargin<4, PRECISION_FLAG=1; end

t00 = clock;
fprintf('\nGCM_EIGANALYSIS...\n')
S = size(P,1);
K = min(S-2,mx);
fprintf('   Computing eigenvectors & eigenvalues of P of size %dx%d..',S,S);
% Run GCM_EIG
[ipr,ipt,ipa] = gcm_eig(P);

% Try to make sure that all recurrent cells are found
if PRECISION_FLAG
    for ii=1:10
        % Repeat until no new recurrent cells among the transient cells are found
        % (gcm_eig can be a bit unreliable..)
        
        % - repeat GCM_EIG for cells that are currently identified as transient
        [ipr_new,ipt_new,ipa_new] = gcm_eig(P(ipt,ipt),K);
        % add new states to recurrent states
        ipr = [ipr;ipt(ipr_new)];
        ipt(ipr_new)=[];
        
        % - exit, if no new recurrent cells were found
        if ~length(ipr_new); 
            clear ipr_new ipt_new ipa_new
            break, 
        end 
        
        fprintf(':');
    end
end

ipr = sort(ipr);
ipt = sort(ipt);
fprintf(' ok.\n');

lipa = length(ipa);
lipr = length(ipr);
lipt = length(ipt);

fprintf('     %d absorbing / %d recurrent states  /  %d transient states',lipa,lipr,lipt);

% Set output arguments / quit?
% ----------------------------
EA.ipa = ipa;
EA.ipr = ipr;
EA.ipt = ipt;


% ====================================
% ==== COMPUTE P_T2R =================
% ====================================
if nargout>1
    fprintf('\n   Computing probabilities of transition to recurrent states...');
    % Decomposed system: P = [P_rr P_rt; 0 P_tt]
    %-------------------------------------------
    % decompose into (P_rr,) P_rt, P_tt
    %P_aa = P(ipr,ipr);
    
    % Try to save memory space..
    % P_rt = P(ipr,ipt);
    m = find(P(ipr,ipt)>0);
    P_rt = spalloc(lipr,lipt,length(m));
    P_rt(2:lipr,:) = P(ipr(2:lipr),ipt);
    P_rt(1,:) = P(ipr(1),ipt);
    
    % P_tt = P(ipt,ipt);
    m = find(P(ipt,ipt)>0);
    P_tt = spalloc(lipt,lipt,length(m));
    P_tt(2:lipt,:) = P(ipt(2:lipt),ipt);
    P_tt(1,:) = P(ipt(1),ipt);
    
    % - clear space for memory
    clear m P
    
    % Probability for entering from a transient cell 
    % to a recurrent cell: P_t2r(ipr,ipt)
    % ----------------------------------------------
    I = speye(lipt);
    ie = find(sum(P_rt)>0);     % find nonzero columns in P_rt (the 0-cells map only to other transient cells)
    try
        % Option 1: try inverting P_t2r = P_rt*inv(1-P_tt)
        % (not likely to work for large P)
        fprintf('\n     P_t2r: Computing inverse..'); 
        invip = inv( (I-P_tt)+100*eps*I );
        fprintf('. ok.')
    catch
        % Option 2: Construct an approximated cumulative sum of multiplications P_t2r = P_rt * sum(P_tt^k), for k=0...inf.
        % In constructing the sum of powers, the problem is that the matrix power becomes a very large matrix 
        % => out of memory. 
        % Below the problem is solved by approximating the matrix power 
        % by a matrix from where small elements are removed, while preserving the sum of columns.
        %
        % The resulting algorithm is very slow and the approximation error is cumulative!
        clear I
        tmp = speye(lipt);          % P_tt^0
        invip = spalloc(lipt,lipt,ceil(lipr/2*lipt));
        k = 0; kmax = 200; critmax = 0.01;  % kmax - max time steps; critmax - max %/100 error
        LIM = 1e9/8;    % Assume memory for a 1MB, 8-bit floating pnt number
        while 1
            %T0 = clock;
            % tmp := P_tt^(k-1)
            invip(ie,:) = invip(ie,:) + tmp(ie,:);
            %T1 = clock;
            while 1
                try
                    tmp = P_tt*tmp; % P_tt^k, k=1,2,... probability of transition from a transient cell to another transient cell at the k'th step
                    break;
                catch
                    % if above fails, assume that it was due to out-of-memory:
                    % decrease LIM and try again..
                    if LIM<100e6, error('GCM_EIGANALYSIS: Out of memory?'); end
                    LIM = ceil(LIM/2);
                    fprintf('\n            switching to %d nonzeros',LIM);
                    tmp = gcm_squeezep(tmp,LIM);    % remove small elements so that max LIM elements are nonzero
                end
            end
            %T2 = clock;
            tmp = gcm_squeezep(tmp,LIM);    % remove small elements so that max LIM elements are nonzero
            %T3 = clock;
            k = k+1;
            % - disp user info
            crit = full(mean(sum(tmp)));
            criti = full(round(100*mean(sum(tmp))));
            if (k==1 | (~rem(k,5) & k<=25) | ~rem(k,25))
                if k==1, fprintf('.failed => approximating using ~%d nonzeros..',LIM); end   % "full inverse failed"
                fprintf('\n            k=%d/%d, %d%% of transitions covered..',k,kmax,100-criti);
                t0 = clock;
            else
                fprintf('.')
            end
            if crit<=critmax, fprintf('\n            Success at k=%d, %3.2f%% ok.',k,100*(1-crit)); break; end % exit if error is less than 'critmax' 
            if k>=kmax, fprintf('\n            Max # iterations exceeded at k=%d, %3.2f%% covered.',k,100*(1-crit)); break; end % exit if # iterations 'kmax' exceeded
            %T4 = clock;
            %[etime(T1,T0) etime(T2,T1) etime(T3,T2) etime(T4,T3)]
            %if k>10, break; end
        end
    end
    % - clear space for memory
    clear I P_tt
    
    P_t2r = P_rt(:,ie)*invip(ie,:); 
    % Sum of columns of P_t2r is not always 1: 
    % this seems to coincide with the cases when 
    % some of the recurrent states are not found (due to use of eigs?)
    % Produce a warning:
    pc_trouble = 1-sum(sum(P_t2r))/lipt;   % this many transitions are 'lost' (transient states which do not map anywhere!)
    if pc_trouble>1e-3
        %fprintf('\n     %4.2f%% inconsistency in computing P_t2r.\n     Increase K (K=%d) or set PRECISION_FLAG (PF=%d).',100*pc_trouble,K,PRECISION_FLAG);
        fprintf('\n     %4.2f%% inconsistency in computing P_t2r.',100*pc_trouble);
    end
end

% ====================================
% ==== COMPUTE E{t_T2R} ==============
% ====================================
if nargout>2
    % Expected recursion time: Et_t2r
    % --------------------------------
    %Et_t2r = P_at*((I-P_tt)+100*eps*I)^(-2); 
    % - this requires a lot of memory for large invip's
    Et_t2r = P_rt(:,ie)*(invip(ie,:)*invip);
    
    % - mean expected-recursion-time
    % - max expected-recursion-time
    if length(Et_t2r)
        mEt_t2r = mean(Et_t2r')';   % average
        maxEt_t2r = max(Et_t2r');   % slowest exp. transition
    else
        mEt_t2r = NaN;
        maxEt_t2r = NaN;
    end
    
    % Set output arguments
    % --------------------
    Et_stat.meanEt = mEt_t2r';
    Et_stat.maxEt = maxEt_t2r;
end

fprintf('\n   ... (%s) ok.\n',gcm_sec2time(etime(clock,t00)));

