function [s,xdk,p_x,x_e] = gcm_estimate_x(xmk,Plant,ymk,a,p_x)
% GCM_ESTIMATE_X    Estimate plant state
%   [s,xdk,p_x,x_e] = gcm_estimate_x(xmk,Plant,<ymk>,<a>,<p_x>)
% s     s(k) state index
% xdk   estimated x(k), in discrete space
% p_x   p_x(k), current state pdf
% xmk   measured x(k)
%
% If third and more arguments are given, a Bayes estimate is constructed
% ymk   measured y(k)
% a     a(k-1), past action
% p_x   p_x(k-1), past state distribution
% x_e   estimated x(k), in continuous space

% Enso 11/2006

if nargin==2
    % use measured state
    s = gcm_x2s(xmk,Plant.Xref);    
    S = size(Plant.Xref,2);
    p_x_meas = spalloc(S,1,3); p_x_meas(s)=1; 
    p_x = p_x_meas;
else    % use estimated state
    % state estimation:
    % p_x(k) ~ l_m(k)*P*p_x(k-1)
    % - output (measured)
    [m,ydk] = gcm_estimate_y(ymk,Plant);
    
    % update estimate (Bayes estimate)
    % - using a and p_x from k-1
    S = size(Plant.Xref,2);
    p_x_est = Plant.h.L(m,:)'.*(Plant.f(a).P*p_x);
    p_x_est = p_x_est/sum(p_x_est); % normalize
    if sum(p_x_est)<100*eps
        % there's a mismatch: according to models this is not possible
        SOL=2;
        switch(SOL)
        case 1
            % Solve by assuming that measurement is false
            str = sprintf('GCM_X_ESTIMATE: Warning: L does not support mapping from p_x to m=%d',m);
            disp(str); disp('L overruled, using p_x.')
            p_x_est = Plant.f(a).P*p_x;
        case 2
            % Solve by assuming that model is false
            str = sprintf('GCM_X_ESTIMATE: Warning: L does not support mapping from p_x to m=%d',m);            
            disp(str); disp('p_x overruled, using L.')
            p_x_est = Plant.h.L(m,:)';
            p_x_est = p_x_est/sum(p_x_est);
        end
    end    
    p_x = p_x_est;  % estimate of p_x(k)
    
    % Infer an estimate from distribution, x_e
    % Find index to closest match of x_e in discretized space, s_e
    ETYPE = 2;
    switch(ETYPE)
    case 1
        % maximum-likelihood-based state estimate
        % ------------------
        s_e = find(p_x==max(p_x)); s_e=s_e(1);
        x_e = Plant.Xref(:,s_e);   % estimated state (in continuous space)
    case 2
        n_x = size(Plant.Xref,1);
        % mathematical expectation
        % ------------------------
        for jj=1:n_x
            x_mean(jj,1) = Plant.Xref(jj,1:S-1)*p_x(1:S-1);
        end
        % discretize estimate
        s_e = gcm_x2s(x_mean,Plant.Xref);
        x_e = x_mean;   % estimated state (in continuous space)
    case 3
        % median
        % ------
        n_x = size(Plant.Xref,1);
        ii = find(p_x>0.1*max(p_x)); % only probs with significant p
        %if length(ii)==0, keyboard; end
        for jj=1:n_x
            x_median(jj,1) = median(Plant.Xref(jj,ii));
        end
        % discretize estimate
        s_e = gcm_x2s(x_median,Plant.Xref);
        x_e = x_median;   % estimated state (in continuous space)
    end
    
    % set s_e as the estimate to be used    
    s = s_e;
end
% State xdk corresponding to estimate index s
xdk = Plant.Xref(:,s);  % estimated discrete state
