function [zz,Sim] = gcm_simr(i1,i2,i3,i4,i5,i6)
% GCM_FREEK  Simulate the original model 
% from initial state to steady-state
%
%   xx = gcm_freek(Plant,x0,a,<T>,<ilist>)
%   xx = gcm_freek(Plant,Cost,x0,q,<T>,<ilist>)
% or
%   xx = gcm_freek(Plant,s0,a,<T>,<ilist>)
%   xx = gcm_freek(Plant,Cost,s0,q,<T>,<ilist>)
%
% The same closed loop simulation can be done using GCM_SIM 
%   [xx,Sim] = gcm_freek(..)
% This M-file has partly similar syntax as GCM_FREE.

% Enso 11/2006
CLOSED_LOOP = 0; if isstruct(i2), CLOSED_LOOP = 1; end
kmax = 100;
if ~CLOSED_LOOP
    Plant = i1;
    if length(i2)==1 & rem(i2,1)==0, zz = Plant.Xref(:,i2);          % input is an index
    else, zz = i2;            % input is the initial state
    end
    q = i3;
    if nargin>3, T = i4; 
    else, T = kmax*Plant.Ts; 
    end
    if nargin>4, ilist = i5; 
    else, ilist=[1:size(Plant.Xref,1)]; 
    end
else
    Plant = i1;
    Cost = i2;
    if length(i3)==1 & rem(i3,1)==0, zz = Plant.Xref(:,i3);          % input is an index
    else, zz = i3;            % input is the initial state
    end
    q = i4;
    if nargin>4, T = i5; 
    else, T = kmax*Plant.Ts; 
    end
    if nargin>5, ilist = i6; 
    else, ilist=[1:size(Plant.Xref,1)]; 
    end
end

% from real time to samples
if length(T)==0
    k0 = 0;
    kf=[];
elseif length(T)==1, 
    k0 = 0; 
    kf = ceil(T/Plant.Ts);
elseif length(T)==2, 
    k0 = floor(T(1)/Plant.Ts); 
    kf = ceil(T(2)/Plant.Ts);
else
    k0 = floor(T(1)/Plant.Ts);
    kf = ceil(T(length(T))/Plant.Ts);
end

funf = Plant.funf;
Ts = Plant.Ts;

[ISGRID,In] = gcm_check_for_grid(Plant.Xref);
SMALL = 1e-6;
k = 1; s=[]; a=[]; u=[];
fprintf('Response to step %d:',q);
fprintf('\n     k     s   a    u\n---------------------------')
while 1
    if ~CLOSED_LOOP
        u = Plant.Uref(:,a);
    else
        s_old = s; a_old = a; u_old=u;
        if ISGRID, s = gcm_x2s(zz(:,k),Plant.Xref,In);
        else, s = gcm_x2s(zz(:,k),Plant.Xref);
        end
        % disp changes in control/state
        a = Cost(q).Pi(s);
        u = Plant.Uref(:,a);
        if (s~=s_old | u~=u_old | a~=a_old) 
            fprintf('\n   %3d %5d %3d',[k s a]);           
            fprintf(' %7.2f',u);
        else
            fprintf('.');
        end
    end
    % simulate one step ahead
    zz(:,k+1)=feval(funf,zz(:,k),u,Ts);
    % check
    s1 = gcm_estimate_x(zz(:,k),Plant);
    s2 = gcm_estimate_x(zz(:,k+1),Plant);
    zz1 = zz(:,k);
    zz2 = zz(:,k+1);
    % check for observed transitions
    cou = Plant.f(a).P(s2,s1);
    if cou==0
        fprintf(' Previously unobserved transition at k=%d  s:%d->%d, a:%d (p=%f)',k,s1,s2,a,full(cou))
    end

    
    % termination
    dzz = max(abs(zz(:,k+1)-zz(:,k)));
    if k>=kf %dzz<SMALL & k>=kf
        break; 
    end
    
    k = k+1;
end

if nargout==0
    [n_z,kf] = size(zz);
    SP = ceil(length(ilist)/2);
    kk = Ts*[0:kf-1]';
    ii = 0;
    for i=ilist
        ii = ii+1;
        if length(ilist)>1, subplot(SP,2,ii); end
        stairs(kk,zz(i,:));
        ytick = gcm_get_partitions(Plant.Xref(i,:)); ytick = ytick(isfinite(ytick));
        set(gca,'ytick',ytick);
        ax = axis;
        axis([ax(1:2) min(min(ytick),ax(3)) max(max(ytick),ax(4))])
        %str = sprintf('x_{%d}',i);
        str =sprintf('%s',Plant.Xname{i});
        ylabel(str);
        xlabel(sprintf('t (T_s=%f)',Plant.Ts))
    end
    gcm_ff;
end

if nargout>1
    % Sim-structure for the same simulation
    S = length(Plant.f(1).P);
    Sim.WW = Plant.Wref(a)*ones(ceil(max(T)/Plant.Ts),1);
    Sim.p_x0 = spalloc(S,1,S); Sim.p_x0(s)=gcm_x2s(zz(:,1),Plant.Xref);
    Sim.x0 = zz(:,1);
    Sim.y0 = NaN; % !
end