function Plant=gcm_generate_model(Plant)
% GCM_GENERATE_MODEL  Generate cell maps by sampling a plant model. Supports vectorized ode.
%
% Generate a plant GCM by simulating the plant model 
% for each cell and counting statistics:
%   Plant=gcm_generate_model(Plant)
% For the first time, center mapping is used (SCM). 
% For consequtive times, states are drawn randomly 
% from a uniform distribution on the support of the each cell.
% The GCM is constructed cumulatively.
% Uses Plant fields: Xref, Uref, Yref, funf and funh
% Sets Plant fields f and h (fields Nsum, P and L)
      
% Enso 11/2006
% rewritten 8/2007: N replaced by Nsum to save memory
% rewritten 11/2007: removed possibility for closed-loop update

t000 = clock;

% Initialize computations
% =======================
%Wref = Plant.Wref;
Yref = Plant.Yref;
Xref = Plant.Xref;
Uref = Plant.Uref;
funf = Plant.funf;
funh = Plant.funh;
Ts = Plant.Ts;
[S,A,Q,M,n_x,n_u,n_y,n_w] = gcmdisp(Plant);

% Check if a previous plant model exists
if ~isfield(Plant,'f'), FIRSTTIME=1; else, FIRSTTIME=0; end

% Reserve space for sparse matrices
% ---------------------------------
if FIRSTTIME
    % reserve spaces for sparse matrices in f and h
    % - # nonzero elements in SxS matrix
    % SS = ceil(min(S,sqrt(S)))*S;
    nzmax_coeff = floor((1/2)*1e9/(S*A*12*2)); % assume 1Gb memory, use half of it
    nzmax_coeff = max(2,min(min(S,6),nzmax_coeff));    % 2<=nzmax<=min(6,S)
    SS = min(S,nzmax_coeff)*S;          
    fprintf('\nGCM_GENERATE_MODEL: Initializing %d sparse %dx%d matrices\n   with %d nonzeros..',A,S,S,SS)
    Nsum = zeros(S,1);
    P = spalloc(S,S,SS);
    for a=A:-1:1, 
        Plant.f(a).Nsum = Nsum;
        Plant.f(a).P = spalloc(S,S,SS);
    end
    MS = min(M,nzmax_coeff)*S;
    Plant.h.Nsum = Nsum; 
    Plant.h.L = spalloc(M,S,MS); 
    PhNsum = Nsum;  % pseudo Plant.h.N for speed
    fprintf('.ok')
else
    PhNsum = Plant.h.Nsum;
    Nsum = Plant.f(1).Nsum;
    P = Plant.f(1).P;
end

% Display info to command window
% ------------------------------
nn = min(Plant.f(1).Nsum)+1; 
switch(rem(nn,10)), 
case 1, str1 = sprintf('%dst',nn); 
case 2, str1 = sprintf('%dnd',nn); 
case 3, str1 = sprintf('%drd',nn); 
otherwise, str1 = sprintf('%dth',nn); 
end; 
if any(rem(nn,100)==[11 12 13]), str1 = sprintf('%dth',nn); end

fprintf('\nGCM_GENERATE_MODEL: generating %s CFMC model',str1);
if A==1, DSP=0; DSP2 =0; else, DSP=1; DSP2 = 1; end

% Sink cell: Smax is the number of states without sink cell
FLAG_SINK = 0; if ~all(all(isfinite(Xref))), FLAG_SINK = 1; end;
if FLAG_SINK, Smax = S-1;  
else, Smax = S; 
end

% Initialize grid search computations
% -----------------------------------
% Check if Xref is from a grid
[GRIDSPACE_Xref,In]=gcm_check_for_grid(Xref);
if GRIDSPACE_Xref & (FIRSTTIME)
    str=sprintf(''); for j=1:length(In), vv=length(In{j}); str=sprintf('%s%dx',str,vv); end; str = str(1:length(str)-1);
    fprintf('\n   %s grid structure assumed for Xref (S=%d).',str,Smax); 
end
% - same for Yref
[GRIDSPACE_Yref,InY]=gcm_check_for_grid(Yref);

% Evaluate samples and count transitions
% ======================================
t00 = clock;
EVALUATEAEQ1ONLY = 1;       % generate random x-points and update L only for a=1
for a=1:A
    
    % PHASE 1 - set initial states 'XfromB' and controls 'UB'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if DSP, 
        fprintf('\n   Simulating plant model %s, %s..',funf,funh); 
        fprintf('\n   -Setting xfrom...'); 
    end
    t0 = clock; t1 = clock;
    
    % set controls 'UB'
    if FLAG_SINK, UB = Uref(:,a*ones(1,S-1));
    else,  UB = Uref(:,a*ones(1,S));
    end
    
    % set initial states 'XfromB'
    if EVALUATEAEQ1ONLY % use same initial state for all control actions
        if FIRSTTIME
            % use reference points, except for sink cell
            if FLAG_SINK, XfromB = Xref(:,1:S-1);
            else, XfromB = Xref(:,1:S);
            end
        else
            % random value from support
            if GRIDSPACE_Xref
                if FLAG_SINK, XfromB = gcm_sampleu(Xref(:,1:S-1),In);
                else, XfromB = gcm_sampleu(Xref,In);
                end
            else
                % if Xref is not grid-based, we should
                % a) generate a random sample
                % b) find the corresponding discretized state
                % c) repeat until one sample for each state is obtained
                % Very slow!
                st = std(Xref')';   % some guess of basis width
                for s=Smax:-1:1
                    xfrom0 = Xref(:,s);
                    xfrom = xfrom0 + st.*randn(n_x,1);  % Gaussian, around xfrom
                    while gcm_x2s(xfrom,Xref)~=s
                        xfrom = xfrom0 + st.*randn(n_x,1);
                    end
                    XfromB(:,s) = xfrom;
                end
            end
        end
        if DSP, str = gcm_sec2time(etime(clock, t1)); fprintf('(%s) ok.\n',str); end
    end
    
    % PHASE 2 - solve state equations 'Xto'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % * Usually the most time consuming step!
    % - if all vectors in UB are the same, solve in a batch    
    t2 = clock; t21 = clock;
    if DSP, fprintf('   -Evaluating xto...'); end
    if all(diff(UB')==0), VFLAG=1; else, VFLAG=0; end
    if VFLAG==0     
        % controls are not all the same
        % Solve for each different vector in UB, for a=1:A...
        Ak_active = gcm_get_partitions(Ak); % control actions in use
        lAk_active = length(Ak_active);
        for j=1:lAk_active
            aa = Ak_active(j);
            ii = find(Ak == aa); % indexes of states for which control action is aa
            u = Uref(:,aa);
            XtoB(:,ii) = feval(funf,XfromB(:,ii),u,Ts);
        end
        % Below, code for one-by-one solutions
        %VDISP = 1;
        %for s=S:-1:1
        %    xfrom = XfromB(:,s);
        %    xto = feval(funf,xfrom,UB(:,s),Ts);
        %    XtoB(:,s) = xto;
        %    % disp..
        %    if VDISP & ~(S==s)
        %        ts = etime(clock,t2)/(S-s);
        %        if etime(clock,t21)>5
        %            ttg = ts*s; str = sec2time(ttg);
        %            fprintf('(%s to go)',str);
        %            t21 = Inf*clock; VDISP=0;
        %        end
        %    end
        %end
    else
        % controls are all the same, UB(:,x)=u
        % funf takes column matrix of X(k)'s as its input argument x, and returns column matrix of X(k+1)'s as output
        u = UB(:,1);
        XtoB = feval(funf,XfromB,u,Ts);
    end
    if DSP, str = gcm_sec2time(etime(clock, t2)); fprintf('(%s) ok.\n',str); end

    
    % PHASE 3 - find transition in discrete space
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if DSP, fprintf('   -Counting state transitions...'); end
    t3 = clock;    
    % Find transition in discrete space: from 's' to 'sdot'
    % -----------------------------------------------------
    if ~GRIDSPACE_Xref
        % unordered reference points (works for all sets of Xref, finds closest match)
        %s = gcm_x2s(xfrom,Xref);
        Sdot = gcm_x2s(XtoB,Xref);
    else
        % reference points have a grid (lattice) topology
        % compare each dimension separately (faster)
        Sdot = gcm_x2s(XtoB,Xref,In);
    end   
    if DSP, str = gcm_sec2time(etime(clock, t3)); fprintf('(%s) ok.\n',str); end

    % PHASE 4 - Update state transition probabilities
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if DSP, fprintf('   -Updating state transition probabilities...'); end
    t4 = clock;
    Nsum = Plant.f(a).Nsum;
    pv = gcm_get_partitions(Plant.f(a).Nsum);
    switch(length(pv))
    case 1
        % if # evalutions is the same with all states
        nsum = Nsum(1);
        P_old = Plant.f(a).P;
        Plant.f(a).P = nsum*Plant.f(a).P/(nsum+1);    % update all states        
        for s=1:Smax % except for the sdot-state
            sdot = Sdot(s);
            Plant.f(a).P(sdot,s) = (nsum*P_old(sdot,s)+1) / (nsum+1);  % execpt for the sdot-state
        end
    otherwise
        for s=1:Smax
            nsum = Nsum(s);
            sdot = Sdot(s);
            ss = [[1:sdot-1] [sdot+1:Smax]];
            Plant.f(a).P(ss,s) = nsum*Plant.f(a).P(ss,s)/(nsum+1);    % update all states
            Plant.f(a).P(sdot,s) = (nsum*Plant.f(a).P(sdot,s)+1) / (nsum+1);  % except for the sdot-state
        end
    end
    % Handle sink cell
    if FLAG_SINK
        Plant.f(a).P(S,S) = 1;
    end
    % write Nsum to structure
    Nsum = Nsum+1;  % increase counter of 'from' states
    Plant.f(a).Nsum = Nsum;
    if DSP, str = gcm_sec2time(etime(clock, t3)); fprintf('(%s) ok.\n',str); end
  
    % PHASE 4 - Update likelihood probabilities
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if DSP, fprintf('   -Updating measurement likelihoods...'); end
    t5 = clock;
    % Solve plant measurement equation: 'yfrom'
    % -----------------------------------------
    % Note: this can be done A times while states are updated only once
    if EVALUATEAEQ1ONLY     % do only once for each new random x
        YfromB = feval(funh,XfromB);
        if ~GRIDSPACE_Yref
            Mv = gcm_x2s(YfromB,Yref);
        else
            Mv = gcm_x2s(YfromB,Yref,InY);
        end

        % Update likelihood probabilities
        PhNsum = Plant.h.Nsum;
        pv = gcm_get_partitions(PhNsum);
        switch(length(pv))
        case 1
            nsum = PhNsum(1);
            L_old = Plant.h.L;
            Plant.h.L = nsum*Plant.h.L/(nsum+1);    % update all states
            for s=1:Smax    % and overwrite the sdot-state
                m = Mv(s);
                Plant.h.L(m,s) = (nsum*L_old(m,s)+1) / (nsum+1);  % execpt for the sdot-state
            end
        otherwise
            for s=1:Smax
                nsum = PhNsum(s);
                m = Mv(s);
                mm = [[1:Mv(s)-1] [Mv(s)+1] M];
                Plant.h.L(mm,s) = nsum*Plant.h.L(mm,s)/(nsum+1);    % update all states
                Plant.h.L(m,s) = (nsum*Plant.h.L(m,s)+1) / (nsum+1);  % execpt for the sdot-state
            end
        end
        % Handle sink cell
        if FLAG_SINK
            Plant.h.L(M,S) = 1; % assuming that M'th measurement should be the place to map a sink cell..
        end
        % rewrite pseudo PhNsum back
        PhNsum = PhNsum+1;
        Plant.h.Nsum = PhNsum;
    end

    % display progress
    % ----------------
    if DSP, 
        tf = (A-1)*etime(clock,t00);
        str = gcm_sec2time(etime(clock, t5)); 
        fprintf('(%s) ok. \n   %d map updates ready in %s: a=%d',str,A,gcm_sec2time(tf),a)
        DSP=0; 
    elseif a>1
        fprintf(',%d',a); 
        if rem(a,15)==0, fprintf('\n   '); end; 
    end

end
% - release memory
clear XfromB XtoB

% PHASE 5 - end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 'Analyze' model
% ---------------
for a=1:A
    % - mean # evaluations per state
    MNE(a) = full( sum(Plant.f(a).Nsum) )/S;
    % - sum of off-diagonal terms (transitions to other states)
    SOD(a) = full( sum(sum( Plant.f(a).P )) - sum( diag(Plant.f(a).P) ) )/S;
    %SOD(a) = full(sum(sum( Plant.f(a).P - diag(diag(Plant.f(a).P)) ))/S);
end
%subplot(211); bar(MNE); subplot(212); bar(SOD); shg; drawnow; 
str = gcm_sec2time(etime(clock, t000));
fprintf('   N = [%d,%d], off-diags %3.2f (%s). ok\n',min(MNE), max(MNE),mean(SOD),str)


