function [P_inf,EA] = gcm_pinf(P,a,EA,P_t2r) 
% GCM_PINF  Estimate mapping to a stationary distribution (time average in far future)
%   P_inf = gcm_pinf(P)
%   [P_inf,EA] = gcm_pinf(Plant,a)
% If indexes to recurrent states are known, the either of the two:
%   P_inf = gcm_pinf(P,[],EA,P_t2r)
%   P_inf = gcm_pinf(Plant,a,EA,P_t2r)
% will reduce computational load, where ipr is the list of indexes to recurrent states, and

% P_t2r is the probability of transition from transient to recurrent cell (see GCM_EIGANALYSIS)
% P_t2r = P_inf(EA.ipr,EA.ipt);

% EI 11/2006

if nargin>1
    if length(a)
        % read transfer probabilities from Plant-structure
        P = P.f(a).P;
    end
end

try  
    
    % Straightforward computations (may not work for large P)
    % Compute projection far (1000 steps) ahead, compute average over 100 projections.
    P0 = P^1000;
    %    P0 = gcm_mpower(P,1000); % compute using eig-decomposition: P = real(V*D.^1000/V);
    P_inf = P0;
    k = 1;
    DISP = 0;
    while 1
        k = k+1;
        P0 = P*P0;
        P_inf_old = P_inf;
        P_inf = (1/k)*P0 + (k-1)/k*P_inf;
        if max(max(abs(P_inf_old-P_inf)))<1e-4, break; end
        if k==100, fprintf('GCM_PINF: computing P_inf..'); DISP=1; end
    end
    if DISP, fprintf('. ok.\n'); end

    % Create EA and P_t2r if above works ok, and if they are requested
    if nargout>1
        [EA,P_t2r] = gcm_eiganalysis(P);
    end
    
catch
      
    % Compute eigenvectors/eigenvalues
    % Compute P_inf by projecting recurrent states far and averaging 
    
    if nargin<4
        % find recurrent states
        %EA = gcm_eiganalysis(P);
        [EA,P_t2r] = gcm_eiganalysis(P);
    end
    
    ipr = EA.ipr;
    ipt = EA.ipt;
    
    % RECURRENT STATES
    % ----------------
    % compute a time average of length K of the recurrent mapping P_rr, far in time (1000);
    P_rr = P(ipr,ipr);
    
    P_rr_k = P_rr^1000;   % projection very far
    K = 100;
    P_rr_inf = 0*P_rr;
    for k=1:K
        P_rr_k = P_rr*P_rr_k;
        P_rr_inf = P_rr_inf +(1/K)*P_rr_k;    % time average
    end
    
    % Rest of the stationary mapping is zeros
    S = size(P,1);
    nz = gcm_spsize(P_rr_inf);
    P_inf = spalloc(S,S,nz);
    P_inf(ipr,ipr) = P_rr_inf;        % set nonzeros as they are
    
    % -free memory
    clear P_rr_k P_rr_inf EA
    
    
    % TRANSIENT STATES
    % ----------------
    % P_t2r is the **stationary** transition probability matrix (from transient to recurrent states)
    if  length(ipr) & length(ipt)
        S = max(max(ipr),max(ipt));
        %gcm_spsize(P_t2r)
        LIM = 1e9/8;
        P_t2r = gcm_squeezep(P_t2r,LIM); % remove very small p's (avoid out-of-memory)
        %gcm_spsize(P_t2r)
        % P_inf(ipr,ipt) = P_t2r; 
        ti0 = clock;
        for i=1:length(ipr)
            pt = P_t2r(i,:);
            k = find(pt);
            if length(k), P_inf(ipr(i),ipt(k)) = pt(k); end
            %if etime(clock, ti0)>5, fprintf('%d, ',i); ti0 = clock; end
        end
    end
    
    
end