function pp = gcm_simp(P,p0,N)
% GCM_SIMP  Simulate a probability transition model 
% from initial state to steady-state
%
%   pp = gcm_simp(P,p0,<kf>)
%
% pp    probability trajectories [S,kf+1]
% P     probability transition matrix [S,S]
% p0    initial state [S,1], or index of a sure state p0(idx)=1
% kf    length of simulation steps. If kf is not given, it is determined automatically.
%       kf=0 returns the initial state, kf=1 one step-ahead trajectory, etc.

if nargin<3, N=1000; end
N = N+1;
S = length(P);
if length(p0)==1
    p1 = spalloc(S,1,S);
    p1(p0) = 1;
elseif length(p0)==S
    p1 = p0;
else
    error('GCM_SIMP: Wrong input length')
end

N_USER_DEFINED = 0;
if nargin==3
    if length(N)
        N_USER_DEFINED = 1;
    end
end
if ~N_USER_DEFINED
    N=1000;    % max length
end

pp = spalloc(S,N,round(sqrt(S))*N);
pp(:,1)=p1;
k = 1;
PERIODIC_FLAG=0;
t0 = clock;
while 1
    % simulate one step ahead
    pp(:,k+1)=P*pp(:,k);
    % check for periodic motion
    for l=k:-1:1
        if sum(abs( pp(:,k+1)-pp(:,l) ))<100*eps
            PERIODIC_FLAG=1;
        end
    end
    k = k+1;
    
    % plot
    if etime(clock,t0)>3 & nargout==0
        nz = find(sum(pp')>0);
        plot(pp(nz,1:k)'); gcm_ff;
        t0 = clock;
    end
    
    % stopping criteria
    if ~N_USER_DEFINED
        dp = max(abs(pp(:,k)-pp(:,k-1)));
        if dp<0.00001 | k>N | PERIODIC_FLAG
            if k>10
                %disp(sprintf('dp = %2.2g',full(dp)))
                break;
            end
        end
    elseif k>=N
        break;
    end
end
pp = pp(:,1:min(N,k));

if nargout==0
    % plot
    nz = find(sum(pp')>0);
    h = plot(pp(nz,1:k)'); gcm_ff;
    axis([0 k+1 -0.1 1.1])
    pf = pp(:,k);
    idx = find(pf>0);
    
    % state numbers in fig
    for i=1:length(nz)
        xp = find( pp(nz(i),:)==max(pp(nz(i),:)) ); xp= xp(1);
        yp = pp(nz(i),xp); yp= yp(1);
        ht = text(xp,yp,num2str(nz(i)));
        set(ht,'color',get(h(i),'color'));
    end
    gcm_ff;
end