function pppp = gcm_squeezep(pp,sqlim)
% GCM_SQUEEZEP  Put small elements of pp to zero, ensuring preservation of probability measure.
%
%   pp = gcm_squeezep(pp,<sqlimit>)
%   pp = gcm_squeezep(pp,<LIM>)
%
% pp          matrix of N probability distributions, one in each column [S,N]
% sqlimit     puts elements p(i,i)<sqlimit to zero. Default is sqlim = 1/S, sqlim<=1
% LIM         finds an sqlimit such that there are max LIM nonzero elements in pp, LIM>1
%
% GCM_SQUEEZEP ensures that the sum of each column remains the same.


% Enso 11/2006, column sum 9/2007

% set limit
[S,N] = size(pp);
if nargin<2
    %sqlim = sqrt(S)/S;
    %sqlim = S.^(1/3)./S;
    sqlim = 1.0./S; 
end

% find a sqlimit
if nargin>1
    if sqlim>1
        % iterate a limit that results (roughly) in LIM nonzeros
        % start from mild squeezing, tighten until LIM is achieved
        LIM = sqlim;    % read input argument
        sqlim = 0.1/S;    % starting value
        ii = 0; coef = 1.5;
        while 1
            ii = ii+1;
            % estimatate #nonzeros with current sqlim
            cs = 0; B=1000;
            for b=1:B:N
                c=b:min(b+B-1,N);
                cs = sum(sum(pp(:,c)>sqlim)) + cs;
            end
            %fprintf('\nx %d<%d? %f',cs, LIM, sqlim)
            if cs<LIM, sqlim = sqlim/coef; break; end
            % increase sqlim
            sqlim = coef*sqlim;
        end
    end
end
sqlim0 = sqlim;
sqlim = min(sqlim,0.1);    % avoid ridiculously large limits
if sqlim0~=sqlim, disp(sprintf('GCM_SQEEZEP: sqlim reset from %f to %f',sqlim0,sqlim)); end

% split computations to smaller parts (avoid out-of-memory)
B=5000;
pppp = [];
b=1;

%for b=1:B:N
while 1
    
    % exit
    if b>N, break; end
    
    % batch computations
    c = [b:min(b+B-1,N)];
    ppp = pp(:,c);
    sppp_old = sum(ppp);
    
    % find logical indexes to large elements in pp
    igt0 = (ppp>=sqlim);
    
    sigt0 = sum(igt0);  % ensure that each column has at least one element..
    try 
        nnz = find(sigt0==0 & sppp_old>=sqlim);  % ..but only if sum of probs is not completely insignificant
        igt0(:,nnz)=1;  % spread equally for all states (may cause out-of-memory)
    catch
        for nn = nnz
            if sppp_old(nn)>=sqlim
                try
                    imx = find(ppp(:,nn)==max(ppp(:,nn))); 
                    igt0(imx,nn)=1; % set largest p to 1 (others to zero)
                catch
                    % try a smaller batch
                    b = b-B;
                    B = ceil(B/5);
                    if B<10, 
                        error('GCM_SQUEEZEP: Error in batch computations.'); 
                    end
                end
            end
        end
    end
    % multiply original matrix element-wise (i.e., multiply small elements by zero)
    ppp = igt0.*ppp;
    
    % ensure that sum of each column remains the same (e.g., 1):
    sppp = sum(ppp);
    ineq1 = find(sppp_old~=sppp);
    for s=ineq1
        ppp(:,s)=sppp_old(s)*ppp(:,s)/sppp(s);
    end
    
    pppp = [pppp ppp];    % Sparse matrices can conveniently catted in this way. Using pp(:,idx)-type of assignments (for vector idx's) is very slow.
    b = b+B;            
end




