function [s,lb,ub]=gcm_x2s(x,Xref,In)
% GCM_X2S    Map state X to closest in Xref
%
%   ss = gcm_x2s(xx,Xref)
%
% xx        matrix of state vectors [n_x,N]
% Xref      set of reference vectors [n_x,S]
% ss        index to closest reference vector [1,N]
% Comp squared error vs. each reference vector
% (arbitrary set of reference points)
%
% If Xref is a grid-lattice (built using gcm_inigrid), use the much faster mapping:
%    ss = gcm_x2s(X,Xref,In)
% where In{i} is a vector of partition for the i'th dimension of X.
%
% If any of the elements in Xref is not finite, a sink cell is assumed.
% The sink cell is always the S'th cell. A vector in xx will be mapped to a sink cell if,
% at any dimension of x:
% a) distance to nearest reference > largest distance between sorted references 
%    [non grid-based reference cells)
% b) x < xmin-dxmin or x > xmax+dxmax, where xmin and xmax are the smallest and largest values 
%    within the reference points, and dxmin and dxmax are half the distance to the next closest 
%    references. [for grid-based reference cells].
% Using the following commands:
%   [ss,dX] = gcm_x2s(X,Xref)
%   [ss,lb,ub] = gcm_x2s(X,Xref,In)
% the output arguments return information on these distances: dx is the largest distance, 
% lb and ub are vectors containing lower and upper bounds, xmin-dxmin and xmax+dxmax [n_x,1]

% Enso 11/2006, rewritten using try-catch 8/2007

FLAG_MULTIPLE_Xref = 0;
if nargin>3, FLAG_MULTIPLE_Xref = 1; end

FLAG_GRID_Xref = 0;
if nargin>2, if length(In), FLAG_GRID_Xref = 1; end; end

[n_x,N] = size(x);              % n_x - # inputs in x, N - # input vectors
[n_x,n_Xref] = size(Xref);

if FLAG_MULTIPLE_Xref, S = max(Sref);
else, S = n_Xref;
end

% Convention: state S is the sink cell!
% x belongs to state S if in any dimension of x:
%   grid:       distance of upper/lower partition is greater than 
%               the distance of upper/lower to its inner member
%   non-grid:   distance to any reference cell is greater than
%               max distance among reference cells.
FLAG_SINK = 0; if ~all(all(isfinite(Xref))), FLAG_SINK = 1; end;

% Remove sink cell (Z) from reference cells, do computations, and add it back at the end.
if FLAG_SINK, Xref(:,S)=[]; S = S-1; Z = S+1; end

if FLAG_GRID_Xref
    
    %if x is a matrix, with x's in each column, compute row-wise
    imin = zeros(n_x,N);
    ns = [];
    for j=1:n_x       
        imin(j,:) = find_imin(x(j,:),In{j});
        ns = [ns length(In{j})];
    end
    
    % change base from ns to integers (10's)
    s = gcm_chbase(imin',ns)';  % vectorized change-of-base

    if FLAG_SINK
        % check for mappings to sink cell
        i_sink = zeros(1,N);
        for i=1:n_x     % assume that partitions in In{i} are ordered from smallest to largest!
            if ns(i)>1
                lb(i) = In{i}(1) - (In{i}(2)-In{i}(1))/2;
                ub(i) = In{i}(ns(i)) + (In{i}(ns(i))-In{i}(ns(i)-1))/2;
                i_sink = max(i_sink, x(i,:)<lb(i) | x(i,:)>ub(i));
            else
                lb(i) = -Inf; ub(i) = Inf;
            end
        end
        s(logical(i_sink)) = Z;
    else
        for i=1:n_x
            lb(i) = -Inf;
            ub(i) = Inf;
        end
    end
    
elseif ~FLAG_GRID_Xref

    % This works for all Xref, but is slow and memory consuming..
    
    SSE = zeros(N,S);
    %SSE = gcm_eucl_norm(x,Xref);
      
    for ss=1:N
        % Compare Euclidean distances in I-dimensional space, not including sink cell
        SE = zeros(n_x,S);
        
        for i=1:n_x
            SE(i,:) = (x(i,ss)-Xref(i,:)).^2;   % sq-errors in i'th dimension of x
        end
        % Comp. sum of squared errors
        if n_x>1
            SSE(ss,:) = sum(SE);
        else
            SSE(ss,:) = SE;
        end
    end        
  
    
    % Find index of the closest 
    [tmp,s] = min(SSE');
    
    if FLAG_SINK
        % Sink cell (non-grid case):
        % errors between x and closest reference points
        E = x-Xref(:,s);
        i_sink = zeros(1,N);
        for i=1:n_x
            maxdXref(i) = max(diff(sort(Xref(i,:))));
            if maxdXref(i)>0
                i_sink = max(i_sink,E(i,:)>maxdXref(i));        % check if any error is larger than allowed
            end
        end
        s(logical(i_sink))=Z;   % set to sink cells
    end

    % set 2nd output argument
    if nargout>1, lb = maxdXref; end
    
end

% In case of multiple reference cells..
if FLAG_MULTIPLE_Xref
    s = Sref(s);
end

function imin_row = find_imin(x,V)
% Compute one-dimensional squared error (distance)
% and return index to element in V that has the smallest error.
% x := xx(i,:)
% V := In{i}
p = length(V);   % # partitions for this dimension
N = length(x);      % # states
try
    sqe=zeros(p,N);  % # initialize sq-error matrix
catch
    % if out-of-memory, split to half
    %fprintf('/')
    imin_row = zeros(1,N);
    Nsplit = floor(N/2);
    imin_row(1:Nsplit) = find_imin(x(1:Nsplit),V);
    imin_row(Nsplit+1:N) = find_imin(x(Nsplit+1:N),V);
    return;
end
% if ok, then compute sqe
for j=1:p 
    sqe(j,:) = (x(1,:)-V(j)).^2; % compare all x(i)'s towards j'th partition (for i=1)
end; 
[tmp,imin_row] = min(sqe);    % find smallest error among j's
