function BofA_proj=gcm_bofa(i1,i2,ixlist,q)
% GCMBOFA   Analyse basins-of-attraction, projected towards state dimensions
%
%   BofA = gcmbofa(Plant,Cost)
%   BofA = gcmbofa(Plant_cl)
% where BofA(i) is a structure with fields:
%   BOFA(q,:)   is the size of basin-of-attraction for set point q, 
%               projected to i'th dimension of x
%   S(q)        sum of size of basins-of-attraction, for 1'th set point
%   In          discretization for i'th dimension of x
%
% The following commands will plot info on basin-of-attraction
%
%   gcmbofa(BofA)
%   gcmbofa(BofA,[],ixlist,<iqlist>)

if isfield(i1,'BOFA')
    % assume that input is a BofA_proj-matrix
    % Plot it.
    BofA_proj = i1;
    if nargin<3, ixlist=[]; end
    if nargin<4, q = []'; end
    gcm_bofa_plot(BofA_proj,ixlist,q);
    clear BofA_proj
    return;
end

if nargin==1
    Plant_cl = i1;
elseif nargin>1
    if length(i2)
        % generate closed-loop maps
        Plant_cl = gcmclosed(i1,i2);
    else
        Plant_cl = i1;
    end
end

% list of projections of x to consider (default: all)
n_x = size(Plant_cl.Xref,1);
if nargin<3, ixlist = [1:n_x]; end
if length(ixlist)==0, ixlist = [1:n_x]; end

% list of set points to consider (default: all)
Q = size(Plant_cl.Uref,2);
if nargin<3, Qlist = [1:Q]; 
else, Qlist = q; 
end
if length(Qlist)==0, Qlist=[1:Q]; end

iq = 0;
for q=Qlist
    % P - closed loop map for the q'th set point
    P = Plant_cl.f(q).P;
    S = length(P);
    K = 100;
    % compute eigenanalysis for the map P
    PrecisionFlag=1;
    [EA,P_t2r] = gcm_eiganalysis(Plant_cl,q,K,PrecisionFlag);
    ii = 0;
    
    for i=ixlist 
        % projection of stats to i'th dimension of x
        xproj = Plant_cl.Xref(i,:);
              
        % GET B-OF-A for states of P, projected to xproj
        % ----------------------------------------------
        % * for each projection to x(i), collect results to a field 'setpoint'
        % * setpoint(q,j) gives the size of basin-of-attraction for
        %   q'th set point and j'th discretized value of x(i)
        [bofa,tmp,BofA_proj(i).BOFA(q,:),BofA_proj(i).In] = gcm_bofa_size(P,EA,P_t2r,xproj);
        BofA_proj(i).S(q,1) = sum(bofa);
        %[BofA,P_rr_inf,BofA_proj(ii).setpoint(iq,:),In] = gcm_bofa(P,EA,P_t2r,xproj);
        BofA_proj(i).Xname = Plant_cl.Xname{i};
    end
end
%keyboard
shg;

if nargout>1
    for i=ixlist
        IN{i} = gcm_get_partitions(Plant_cl.Xref(i,:));
    end
end


% -----------------------------------------
function gcm_bofa_plot(BofA,ixlist,qlist)
% PLOT

n_x = length(BofA);
Q = size(BofA(1).BOFA,1);

if nargin<2, ixlist = [1:n_x]'; end
if length(ixlist)==0, ixlist=[1:n_x]; end
if nargin<3, qlist = [1:Q]; end
if length(qlist)==0, qlist=[1:Q]; end

%ixlist=[]; for i=1:n_x, if length(BofA(i).BOFA), ixlist = [ixlist,i]; end; end

L = ceil(length(ixlist)/2);
ii = 0;

for i=ixlist
    ii = ii+1;
    In = BofA(i).In;
    BOFA = BofA(i).BOFA;
    S = max(BofA(i).S);
    
    if length(In)>1

        xx = In;
        yy = 100*BOFA(qlist,:)'/S;   
        
        lix = length(ixlist);
        switch lix
        case 1
            ;
        case 3
            subplot(1,3,ii);
        otherwise
            subplot(L,2,ii); 
        end

        % bar-plot
        H=bar(xx,yy);
        cb = [1 0 0;0 1 0;0 0 1;0.5 0.5 1;0.5 1 0.5;1 0.5 0.5;0 0 0];
        for j=1:length(H)
            c = min(7,cb(j,:));
            set(H(j),'FaceColor',c,'EdgeColor',0.75*c);
        end

        xtick = gcm_get_partitions(In);
        set(gca,'xtick',xtick);

        if isfield(BofA,'Xname'), xlabel(sprintf('%s',BofA(i).Xname));
        else, xlabel(sprintf('x_%d',i));
        end

        ylabel('# b-of-a (%)');
        
        dx = (max(In)-min(In))/length(In) + 1e-6;
        text(max(In)+dx,100,sprintf('\\Sigma %g ',S),'hor','r','ver','top');
        
        for iii=1:length(qlist), M{iii} = sprintf('%d',qlist(iii)); end
        h=legend(M,-1);
        set(get(h,'title'),'string','set point');
        
        colormap('colorcube');
        axis([min(In)-dx max(In)+dx -05 105]);

        
    else
        ;
    end    
end




