function Plant_cl = gcmclosed(Plant,Cost,q)
% GCM_CLOSED    Generate a GCM map for a closed loop system
% using Plant GCM model for plant simulation:
%
%   Plant_cl = gcmclosed(Plant,Cost)
%
% Plant_cl will be a system with Q setpoints as inputs and M outputs.
%
% Following substitution is made:
%       Plant_cl.Uref <- Plant.Wref
%       Plant_cl.f(q).N(:,s) = Plant.f(a).N(:,s);   
% with 'a' from  Cost(q).Pi(s). The closed-loop GCM mapping
%       Plant_cl.f(q).P     
% is computed using plant model (in Plant) and given controller (Cost(q).Pi).

% Enso 11/2006

[n_x,S] = size(Plant.Xref);
[n_u,A] = size(Plant.Uref);
[n_y,M] = size(Plant.Yref);

Q = length(Cost);
if nargin>2
    Qlist = q;
else
    Qlist = [1:Q];
end

fprintf('\nGCM_CLOSED: Building %d closed-loop GCM maps\n',Q);

% Initialize maps
% Copy initial info from open-loop map 'Plant' to closed-loop map 'Plant_cl'
Plant_cl.Xref = Plant.Xref;
Plant_cl.Yref = Plant.Yref;
Plant_cl.Uref = Plant.Wref;
Plant_cl.Wref = Plant.Wref;
Plant_cl.funf = Plant.funf;
Plant_cl.funh = Plant.funh;
Plant_cl.simf = Plant.simf;
Plant_cl.simh = Plant.simh;
Plant_cl.Ts = Plant.Ts;
Plant_cl.Xname = Plant.Xname;
Plant_cl.Yname = Plant.Yname;
Plant_cl.Uname = Plant.Wname;
Plant_cl.Wname = 'closed-loop system';

% Reserve space for closed-loop measurement info
nzmax_coeff = 5;  % # nonzeros in sparse matrices = nzmax_coeff*S
MS = min(M,nzmax_coeff)*S;
MS = ceil(min(M,sqrt(M)))*S;
Plant_cl.h.Nsum = zeros(S,1); 
Plant_cl.h.L = spalloc(M,S,MS); 
for q=Qlist
    % Reserve space for closed-loop state transtion info
    SS = min(S,nzmax_coeff)*S;        
    Plant_cl.f(q).Nsum = zeros(S,1);
    Plant_cl.f(q).P = spalloc(S,S,SS);
    %P = spalloc(S,S,SS);
end

FDSP = 1;
for q=Qlist
    if FDSP, fprintf('   q='); FDSP=0; end
    fprintf('%d,',q);
    
    % Simple version (the faster version -organized using a- was not faster)
    for s=1:S
        % set plant control
        a = Cost(q).Pi(s);
        % propagate plant forward from state s: distribution at s'
        p_xdot = Plant.f(a).P(:,s);
        % Set statistic
        % sdot = find(p_xdot);
        % Plant_cl.f.N(sdot,s)=1;
        % Set transtion matrix
        Plant_cl.f(q).P(:,s) = p_xdot;
        Plant_cl.f(q).Nsum(s) = Plant.f(a).Nsum(s);
    end
    Plant_cl.h.L = Plant.h.L;
    Plant_cl.h.Nsum = Plant.h.Nsum;
end

fprintf(' ok.\n');