function Cost = gcmpredpol(Plant,Cost,q)
% GCMPREDPOL Compute optimal policy for predictive control
%
%   Cost = gcmpredpol(Plant,Cost,q)   for q'th set point
%   Cost = gcmpredpol(Plant,Cost)     for all set points
%
% reads:
%   Cost(q).H_p   prediction horizon [1,1]
%   Cost(q).R     (immediate) costs [S,A]
%   Plant.f(a).P  probability transition matrices for each a=1,2,...,A [S,S]
% sets:
%   Cost(q).Pi    control policy [S,1]
%   Cost(q).Jstar optimal cost (in prediction horizon) [S,1]

t00 = clock;

% solve states for prediction horizon
[S,A] = size(Cost(1).R);
Q = length(Cost);

% set list of q's (set point indexes)
if nargin<3, qlist = [1:Q]; 
else qlist = q;
end

%J = zeros(S,lql*A);     % initialize cost-matrix
for q=qlist, tmp(q).J = zeros(A,S); end     % initialize temporary cost-matrices
maxH_p = max([Cost(qlist).H_p]);
str = sprintf('%d,',qlist); str = str(1:length(str)-1);
fprintf('\nGCM_PREDPOL: Evaluating set points q=%s for H_p = %d\n   ',str,maxH_p)

CNT = 1; t0 = clock; ss = 0;
% Compute state trajectories (control horizon H_c=1)
for a=1:A

    % compute policy for each state: set initial states to 1's 
    pp = speye(S,S);  % Hox: fast but does not reserve space..
    %pp = spalloc(S,S,6*S);  for i=1:S, pp(i,i)=1; end
    % compute trajectory in prediction horizon
    for h=0:maxH_p
        % evaluate costs at k+h, for each set point q and action a
        for q=qlist
            if Cost(q).H_p>=h   % q's may have different H_p's
                %J(:,ia) = J(:,ia) + pp'*Cost(q).R(:,a);     % sum of (immediate) costs in prediction horizon
                tmp(q).J(a,:) = tmp(q).J(a,:) + Cost(q).R(:,a)'*pp;     % sum of (immediate) costs in prediction horizon
            end
        end
        % quit
        if h==maxH_p, break; end
        
        % Squeeze size of pp, i.e. set small elements to zero (to avoid out-of-memory).
        % LIM is an experimental limit!!! For another computer, another value may be needed. Use as large value as possible.
        % This will modify results (!), but is needed when horizons are long, and p become flat.
        % Experimental results with S=34992:    percentage different control actions vs. no squeezing
        %   sqlim = sqrt(S)/S                   11.0 %
        %   sqlim = S.^(1/3)./S                  4.6 %
        %   sqlim = 1.0./S                       0.4 %
        %   sqlim = 1.0./S + using LIM           0.1 %
        LIM = 1.0e6;     
        if gcm_spsize(pp)>LIM   
            %s1 = spsize(pp);
            %if a==11 & h==3, keyboard; end
            pp = gcm_squeezep(pp,LIM);
            fprintf('')
            %s2 = spsize(pp);
            %disp([a h s1 s2]);
        end
        %fprintf('\n%d %d %d',a ,h, spsize(pp))
        % compute state pdf at k+h+1 using action a     
        pp = Plant.f(a).P*pp;   
    end
    
    % disp
    if CNT<4 | etime(clock, t0)>15
        if rem(CNT,10)==0, fprintf('\n   '); end
        CNT = CNT+1;
        fprintf(' %d%%',round(100*a/A) );
        t0 = clock;
    end
    
end

% find optimal actions
for q = qlist
    [Jmin,Pi] = min(tmp(q).J);
    % store result
    Cost(q).Pi = Pi;
    Cost(q).Jstar = Jmin';
end

str = gcm_sec2time(etime(clock,t00));
fprintf(' 100%% (%s) ok. ',str);
