function Cost = gcmpredpoldp(Plant,Cost,q)
% GCMPREDPOLDP Compute optimal policy for predictive control
%              using dynamic programming
%
%   Cost = gcmpredpol(Plant,Cost,q)   for q'th set point
%   Cost = gcmpredpol(Plant,Cost)     for all set points
%
% reads:
%   Cost(q).H_p   prediction horizon [1,1]
%   Cost(q).H_c   control horizon [1,1]
%   Cost(q).R     (immediate) costs [S,A]
%   Plant.f(a).P  probability transition matrices for each a=1,2,...,A [S,S]
% sets:
%   Cost(q).Pi    control policy [S,1]
%   Cost(q).Jstar optimal cost (in prediction horizon) [S,1]

t00 = clock;

% solve states for prediction horizon
[S,A] = size(Cost(1).R);
Q = length(Cost);

% set list of q's (set point indexes)
if nargin<3, qlist = [1:Q]; 
else qlist = q;
end

for q = qlist 
    % Read controller parameters H_p & H_c
    H_p = Cost(q).H_p;
    if isfield(Cost(q),'H_c'), H_c = Cost(q).H_c;
    else, H_c = 1; %H_p;
    end
    
    % Solve backwards, starting from end-of-horizon
    J = zeros(A,S);
    J_star = zeros(S,1);
    Pi = zeros(H_p,S);
    
    % a) Costs for last decision step: h=H_c...H_p
    for a=1:A   % for each action
        pp = speye(S,S);
        for h=H_c-1:H_p % predict states & evaluate costs to end of horizon
            J(a,:) = J(a,:) + Cost(q).R(:,a)'*pp;
            pp = Plant.f(a).P*pp;
        end
    end
    [J_star,Pi(H_c,:)] = min(J);
    
    % b) Costs before last decision step: h=0...H_c-1
    for h=H_c-2:-1:0
        % cost-to-go(s,a) = immediate cost R(s,a) + cost-to-go(s')
        for a=1:A
            P = Plant.f(a).P;   % probability transition matrix
            J(a,:) = Cost(q).R(:,a)' + J_star*P;
        end
        % find optimal action a
        [J_star,Pi(h+1,:)] = min(J);
    end
    % store results (policy from first step in horizon)
    Cost(q).Pi = Pi(1,:)';
    Cost(q).Jstar = J_star';
end
