function Sim = gcmsim(Plant,Cost,Sim)

% GCMSIM   Run a GCM simulation
%
%   Sim = gcmsim(Plant,Cost,Sim)
%
% Fields in Plant:
%   funf, funh    strings for plant model functions f and h 
%       x(k+1) = f(x(k),u(k),Ts)
%       y(k) = h(x(k))
%   simf, simh    strings for plant simulation functions f and h
%   Xref  state reference points (cell centers) [n_x,n_Xref]
%   sref  corresponding state indexes [1,n_Xref]
%         State vector x is mapped to discrete state sref(i), where i minimizes ||x-Xref(i)||.
%         The number of discrete states is S = max(sref).
%         First S-1 reference points in Xref are the 'representative' cell centers.
%         Cell S is the sink cell: P(S,S)=1 (all space outside of finite region of interest)
%         Introduction of multiple reference points allows precise specification of cell boundaries.
%   Yref  output reference points [n_y,M]
%   Uref  control actions [n_u,A]
%   Wref  set points [n_w,W]
%   f   state model structure vector [A], with fields
%       N    counter of observed transitions [S,S] (sparse)
%            with elements n(sdot,s)
%       P    transition probability matrix [S,S] (sparse)
%   h   measurement model, with fields
%       N    counter of observed transitions [S,S] (sparse)
%            with elements n(sdot,s)
%       L    transition probability matrix [M,S] (sparse)
% 
% Cost  structure-vector of costs [Q]
%   Contains sparse matrices:
%       R       costs for state-action pairs [S,A]
%       Jstar   costs-to-go for actions [S,1]
%       gam     discounting factor (controller tuning param)
%       Pi      optimal control policy/table [S,1]
%
% Sim   structure of simulation parameters
%   WW  simulation set points [K,n_y]
%   x0  plant initial state [n_x,1]
%   y0  plant initial output [n_y,1]
%   p_x0 initial state distribution [S,1]
%
%   UU      control sequence [K,n_u]
%   XX_?    state sequence [K,n_x]
%   YY_?    plant output sequence [K,n_y], where:
%       ?=s     simulated (noiseless, unknown in real life)
%       ?=m     measured, in real space
%       ?=d     discrete, in discrete space (estimated)
%   PDFX    state distribution [K,S]
%   QMSA    [q(k) m(k) s(k) a(k)] [K,4]
%   rmse_rd RMSE on measured and discrete outputs (vs set point)
%   mctg    mean cost to go
 
% naming conventions:
% xsk       simulated states (not known in real life)
% xpk       measured states (known)
% xdk       estimated states, in discrete space

% Closed loop
% ===========
fprintf('\nSimulating a closed loop system..')

Sim.UU=[];   % measured control action
Sim.XX_s=[];   % true state
Sim.XX_m=[];   % measured state
Sim.XX_d=[];   % estimated discrete state
Sim.YY_s=[];   % true output    
Sim.YY_m=[];   % measured output
Sim.YY_d=[];   % estimated discrete output
Sim.PDFX=[];
Sim.QMSA=[];


t0 = clock;
K = size(Sim.WW,1);
for k=1:K
    
    % set recursion
    if k==1
        % initial values
        xsk = Sim.x0; 
        ysk = Sim.y0; 
    else
        % shift in time
        ysk = ysk1;
        xsk = xsk1;
    end
    
    % measure output and state
    ymk = ysk;  % noiseless measurement
    xmk = xsk;  % noiseless measurement
    
    % estimate state and output
    % -------------------------
    XYEST = 1;
    switch(XYEST)
    case 1
        % use measurements
        [s,xdk,p_x] = gcm_estimate_x(xmk,Plant);
        [m,ydk] = gcm_estimate_y(ymk,Plant);
    case 2
        % use a state estimate, followed by an output estimate
        if k==1, p_x = Sim.p_x0; s = find(p_x==max(p_x)); xdk = Plant.Xref(:,s);
        else, [s,xdk,p_x] = gcm_estimate_x(NaN,Plant,ymk,a,p_x);
        end
        [m,ydk] = gcm_estimate_y(ymk,Plant,p_x);
    end
          
    % get setpoint
    wmk = Sim.WW(k,:)';
    q = gcm_x2s(wmk,Plant.Wref);
    
    POL = 1;
    switch(POL)
    case 1
        % Use predetermined policy in Cost.Pi
        % set next control, a(k)
        % - determine using maximum likelihood
        s_c = find(p_x==max(p_x)); s_c=s_c(1);
        a = Cost(q).Pi(s_c);
        % - measure control
        umk = Plant.Uref(:,a);
    case 0
        % Solve predictive controller on-line 
        % starting from distribution p_x
        a = gcmpred_online(p_x,Plant,Cost,q);
        % - measure control
        umk = Plant.Uref(:,a);
    end
        
    %disp(sprintf('q=%d s=%d a=%d -- %7.2f %d %d %d',[q s_c a umk']))
    %S = length(Plant.f(1).P);
    %if s_c==S, keyboard; end

    % simulate plant x(k+1), y(k+1): xsk -> xsk1, ysk1
    usk = Plant.Uref(:,a);
    xsk1 = feval(Plant.simf,xsk,usk,Plant.Ts);
    ysk1 = feval(Plant.simh,xsk1);
    % set state noise
    % -additive noise DX
    % -multiplicative noise PX
    % -forced value FX
    if k<K
        if isfield(Sim,'DX'), fx = Sim.FX(k+1,:)'; j=dx~=0; xsk1(j) = xsk1(j)+dx(j); end
        if isfield(Sim,'PX'), px = Sim.FX(k+1,:)'; j=px~=1; xsk1(j) = xsk1(j).*px(j); end
        if isfield(Sim,'FX'), fx = Sim.FX(k+1,:)'; j=fx~=0; xsk1(j) = fx(j); end;
    end
    
    % log and plot
    %[xsk,xmk,xdk]
    %[ysk,ymk,ydk]
    Sim.UU(k,:)=umk';   % measured control action
    Sim.XX_s(k,:)=xsk';   % true state
    Sim.XX_m(k,:)=xmk';   % measured state
    Sim.XX_d(k,:)=xdk';   % estimated discrete state
    Sim.YY_s(k,:)=ysk';   % true output    
    Sim.YY_m(k,:)=ymk';   % measured output
    Sim.YY_d(k,:)=ydk';   % estimated discrete output
    Sim.PDFX(k,:) = p_x';
    Sim.QMSA(k,:) = [q m s a];

    kk = [1:k]';
    % error in real / digital world
    rmse_m = gcm_rmse( Sim.YY_m(kk,:), Sim.WW(kk,:) );
    rmse_d = gcm_rmse( Sim.YY_d(kk,:), Sim.WW(kk,:) );
    Sim.rmse_rd = [rmse_m rmse_d];
    
    % estimated (discounted) costs-to-go from current state (for optimal control only)
    if isfield(Cost(1),'Jstar')
        Ctg(k,1) = Cost(q).Jstar(s);
        Sim.mctg = mean(Ctg);
    else
        Sim.mctg = NaN;
    end
   
    if etime(clock,t0)>3
        gcm_sim_plot(Plant,Cost,Sim);
        t0 = clock;
    end
end
% plot
gcm_sim_plot(Plant,Cost,Sim);

fprintf('. ok\n')
