function [Sink,Stat] = gcm_stability(i1,i2,sss,Stat)
% GCM_STABILITY Analyse stability of a closed-loop map.
% The system is assumed to be stable, if the probability of ever entering a sink 
% state is 0, where the S'th state is the sink state.
%
%   [Sink,Stat] = gcm_stability(Plant,Cost)
%   [Sink,Stat] = gcm_stability(Plant_cl)
%
% Sink(q,s) is the stationary probability of entering the sink cell, when
%           initially in cell s, and under set point control q. [Q,S]
%           i.e., using transition map Plant_cl.f(q).P
% Stat      is a vector structure. Stat(q) has the following fields:
%           P_inf   the stationary closed-loop map
%           EA      structure, fields are indexes to recurrent and transient states
% Stable states are states with Sink(q,:)==0 (or very small at least).
%
% The following syntaxes will make use of previously computed static maps
%   gcm_stability(Plant,Cost,[],Stat);
%   gcm_stability(Plant_cl,[],[],Stat);
%
% With no output arguments given, a bar-histogram plot of state distributions 
% is produced, one for each set point q. Each bar shows the number of states 
% and the cumulative percentage of (stable) states. The probability of entering
% a sink cell is indicated at the x-axis.
%
% The following commands will plot info on stability:
%
%   gcm_stability(Sink)
%   gcm_stability(Sink,[],ss)
%
% where ss is a list of indexes to states of interest.

% Enso 11/2006

if nargin==1
    Plant_cl = i1;
elseif nargin>1
    if length(i2)
        % generate closed-loop maps
        Plant_cl = gcmclosed(i1,i2);
    else
        Plant_cl = i1;
    end
end
if nargin<3
    if isstruct(i1), S = length(Plant_cl.f(1).P);
    else, S = size(i1,2);
    end
    sss = [1:S];
end

if ~isstruct(i1)
    % Assume that input is a Sink-matrix from some previous run
    % Plot it.
    % =========================================================
    Sink = i1;
    gcm_stab_plot(Sink,sss);
    gcm_ff;
    clear Sink
    return;
end
clear i1 i2

% Build Sink-matrix
% ===================
Q = size(Plant_cl.f,2);
S = length(Plant_cl.f(1).P);
for q=1:Q
    
    % STATIONARY MAPS TO SINK STATE
    % =============================
    % probability transition map for set point q: Plant_cl.f(q).P    
    % stationary map
    % gcm_spsize(Plant_cl.f(q).P)
    if nargin<4
        if nargout==1
            P = gcm_pinf(Plant_cl.f(q).P);
        else
            [P,EA] = gcm_pinf(Plant_cl.f(q).P);
        end
        % stationary map to sink state
        pS = P(S,:);
    else
        % read from input argument
        pS = Stat(q).P_inf(S,:);
    end
    
    % Build the Sink matrix
    % ---------------------
    % Output arguments
    Sink(q,:) = full(pS);
    
    % Build the Stat-structure
    % ------------------------
    % Store for possible later use
    if nargout>1 & exist('P')
        Stat(q).P_inf = P;
        Stat(q).EA = EA;
    end
end

% Plot, if no output arguments
if nargout==0
    gcm_stab_plot(Sink,sss);
    clear Sink
    gcm_ff;
end


% -------------------------------------------------------------
function gcm_stab_plot(Sink,sss)
% Plot info on stability
[Q,S] = size(Sink);
for q=1:Q
    
    % PLOT
    % ====
    % plot probability of mapping state s to sink state
    % categories:
    % p=1
    % 0.9<=p<1
    % 0.5<=p<0.9
    % 0.1<=p<0.5
    % 0<p<0.1
    % p=0
    
%    if Q>1, subplot(Q,1,q); end
    
    % set probabilities of transitions from states not in sss to zero
    pS = zeros(1,S);
    pS(sss) = full(Sink(q,sss));

    % create histogram
    small = 1e-9;
    EDGES=[0 small 0.02 0.5 0.98 1-small 1+small];
    N = histc(pS(sss),EDGES);
    NN(q,:)=N;
    Pros(q,:) = 100*N/sum(N);
end

bar(Pros(:,1:6)');

% text
for i=2:6,
    str = sprintf('%1.0f ',NN(:,i)); str = ['{',str(1:length(str)-1),'}'];
    text(i,max(Pros(:,i)),str,'fontsize',8,'ver','bott','hor','c')
%    if i<4, text(i,Pros(i),sprintf('%d (%1.0f%%)',N(i),100*sum(N(1:i))/sum(N)),'ver','bott','hor','c');
%    else, text(i,Pros(i),sprintf('%d',N(i)),'ver','bott','hor','c');
%    end
end

% axes
txt = {'p=0','0<p<=0.02','0.02<p=<0.5','0.5<p<=0.98','0.98<p<1','p=1'};
for i=1:6
    text(i,-1,txt{i},'hor','c','ver','t','fontsize',8);
end
txt2{1}='stable'; txt2{6}='unstable'; for i=2:5,txt{i}=''; end
set(gca,'xtick',[1:6],'xticklabel',txt2);

% title
title('Probability for entering a sink cell'); 
%text(6.5,S,sprintf('\\Sigma = %d',sum(N)),'ver','top','hor','right')
text(6.5,100,sprintf('\\Sigma = %d ',sum(N)),'ver','top','hor','right')

% legend
for q=1:Q, tmp{q}=sprintf('%d',q); end
hl=legend(tmp,-1);
set(get(hl,'title'),'string','set point q')

%ylabel(sprintf('# (q=%d)',q));
ylabel('# cells (%)')
%axis([0.5 6.5 -1 S+1])
axis([0.5 6.5 -5 105])

    
