function [Gr,P,St,C,Traj]=unravel(C,DISP,Plant)
% Unravelling algorithm 
%
%   [Gr,P,St,C,Traj]=unravel(C,DISP,Plant)
%
%   C   Cell map (binary) [S,1]
%       If input argument C is a SxS probability transition map, 
%       a dominating simple (binary) cell map C is created from it.
%   Gr  Group numbers [S,1]
%   P   Periodicty numbers [S,1]
%   St  Step numbers [S,1]
%   DISP    Display flag
%           DISP=0  no display (default)
%           DISP=1  display a plot of results
%           DISP=2  display a plot of results and a list of sequences 
%                   as searched by the unravelling algorithm.
%   S is the number of cells.
%
%   Traj gives the state indexes for the plotted trajectories
%   - first max(Gr) columns give trajectories with smallest step numbers  
%   - max(Gr) columns that follow give trajectories with the largest step number
%
% Adopted from C.S.Hsu (1987) pp. 148-151 (Sec 8.2.2)

t0 = clock;

Traj = [];
if nargin<2, DISP=0; end

[r,c] = size(C);		% # cells
if r==c
    [e,C] = max(C);     % create a dominating simple cell map
    N = length(C);
    disp('UNRAVEL: Dominating SCM is used..')
else
    N=max(r,c);     
end
Gr = zeros(N,1);	% initialize all cells as virgin cells
P=[]; St=[];
g = 0;			% initial group number counter

for z=1:N		% for each cell
	m=0;		
    if DISP & etime(clock, t0)>5, DISPNOW=1; t0=clock; else, DISPNOW=0; end
	if Gr(z)==0	% if virgin cell
		if DISP>1 | DISPNOW, fprintf('%d',z); end
		Gr(z,1)=-1;	% mark cell as "under processing"
		b = z;		% present cell

		m = m+1;
		b = C(b);	% next cell in the sequence
		if DISP>1 | DISPNOW, fprintf('->%d',b); end

		while(Gr(b)==0)		% (i) while virgin cell
			Gr(b)=-1;
			m=m+1;
			b=C(b);
			if DISP>1 | DISPNOW, fprintf('->%d',b); end
		end

		if Gr(b)==-1
			% (iii) "Assign Gr,P,St numbers (subroutine NEWG)"
			% cell has appeared in the present sequence => periodic motion
			if DISP>1 | DISPNOW, fprintf('->%d (m=%d).\n',b,m); end
			[Gr,P,St,g]=unrav_ng(z,b,m,g,C,Gr,P,St);
        else
			% (ii) "Assign Gr,P,St numbers (subroutine OLDG)"
			% cell has appeared in one of the previous sequences => known character
			if DISP>1 | DISPNOW, fprintf('!\n',b); end
			[Gr,P,St]=unrav_og(z,b,m,C,Gr,P,St);
		end	
	else
		%if DISP, fprintf('%d!\n',z); end
	end
end

G = max(Gr);
if DISP, fprintf('Sorting %d groups...',G); end
% Sort results so that group #1 is the largest, etc.
if 1
    % find  n: sorted list of # elements in a group
    %       i: corresponding group numbers in the unsorted list
    for g=1:G, n(g) = length(find(Gr==g)); end
    [n,i] = sort(n); n=n(G:-1:1); i=i(G:-1:1);
    for g=1:G
        i_old = find(Gr==i(g)); % replace old group number
        Gr_new(i_old,1) = g;    % by the new one
    end
    Gr = Gr_new;
end
if DISP, fprintf('ok.\n'); end
if DISP
    if nargin<=2
        unravel_disp(Gr,P,St,C); 
    else
        unravel_disp(Gr,P,St,C,Traj,Plant); 
    end
end
