function [xdot,o2,o3] = vdvdv5(t,x,u,w)
% Multivariable CSTR Van der Vusse reaction. 
% [tspan,x0] = vdvdiffeq([],[],'init')
% [t,x] = ode23t('vdvdiffeq',[0 Ts],x0,[],<u>,<w>);
% x	process states
%		x(1)	concentration of A
%		x(2)	concentration of B (product, measured)
% 		x(3)	temperature in reactor (measured)
%		x(4)	temperature in cooling jacket
%		x(5)	feed temperature
% If input argument x is of length two, the system is simulated 
% as a two-state system, using nominal values for x(3) and x(4).
% u	process manipulated variable(s)
%		if u is scalar, then feed flow V'/VR is set to u (3<14.19<35)
%		if u is 2x1, V'/VR=u(1); Q'K = u(2) (-9000<-1113.5<0).
% 		u is held constant during the whole simulation.
% feed temperature (v0) is a process disturbance (100<104.9<115)

% For model and benchmark problem, see Chen, Kremling and Allgwer (1995) 
% Nonlinear predictive control of a benchmark CSTR. Proc. ECC'95, Rome, 3247-3252.
% E.I. 10/00. See also VDVPARAM.M.
% Batch update EI 8/2007

% Read parameter values
vdvparam;

% process disturbances
if nargin<4, w=[]; end
switch length(w)
case 0
    ;
case {1,2}
    % input feed temperature 100<v_0<115
    x(5,:) = w(1);
    if length(w)==2
        % parameter uncertainty
        switch w(2)
        case 0
            ;
        case 1
            k_0 = [1.327e12 1.327e12 8.773e9];
            dH_RAB=6.56; dH_RBC=-9.08;dH_RAD=-40.44;
        case 2
            k_0 = [1.247e12 1.247e12 9.313e9];
            dH_RAB=1.84; dH_RBC=-12.92;dH_RAD=-43.26;
        end
    end
end

% manipulated variables
if nargin<3, u=[]; end
switch length(u)
case 0
    ;
case 1
    % single-input case
    Vdot = V_R * u(1); Vdot = min(max(3*V_R,Vdot),35*V_R);   
    Qdot_K = Qdot_K;
case 2
    % multi-input case
    Vdot = V_R * u(1); Vdot = min(max(3*V_R,Vdot),35*V_R);
    Qdot_K = u(2); Qdot_K = min(max(-9000,Qdot_K),0);
end

% convert x from vector to matrix
n = length(x); 
ns = 5; 
x = gcm_v2v(x,ns);    

c_A = x(1,:); 
c_B = x(2,:); 
v = x(3,:); 
v_K = x(4,:); 
v_0 = x(5,:);

% reaction velocities via Arrhenius law
for i=1:3 
    k(i,:) = k_0(i) * exp( E(i)./(v+273.15) ); 
end

% concentration of substrate A
%          in                    A->B         A->D
cdot_A = (Vdot/V_R)*(c_A0-c_A) - k(1,:).*c_A - k(3,:).*c_A.^2;
% concentration of substrate B
%           out             A->B       B->C
cdot_B = -(Vdot/V_R)*c_B + k(1,:).*c_A - k(2,:).*c_B;

% reactor temperature
% change = in-out + reaction - to cooler
vdot = (Vdot/V_R)*(v_0-v) ...
    - (1/(ro*C_p))*( k(1,:).*c_A*dH_RAB + k(2,:).*c_B*dH_RBC + k(3,:).*(c_A.^2)*dH_RAD ) ...
    - (k_w*A_R)/(ro*C_p*V_R)*(0.1).^3.*(v-v_K);			% 0.1^3 due to conversion from litres to m^3
% cooling jacket temperature
%                     heat removal + heat from reactor
% Qdot_K = Vdot/V_R*(20-v_K); % with this trick we get closer to optimum yield at c_B = 14.19
vdot_K = (1/(m_K*C_PK))*(Qdot_K + k_w*A_R*(v-v_K));
% input feed temperature (constant)
vdot_0 = zeros(size(cdot_A));

xdot = [cdot_A;cdot_B;vdot;vdot_K;vdot_0];


% x'

% convert x from matrix to vector 
xdot = gcm_v2v(xdot);


